// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

export { default as SecretShare } from "./src/secretShare";
export { default as Utils } from "./src/utils";
export { default as AESKey} from './src/aesKey'
export { default as C25519Point} from './src/c25519Point'
export { default as ed25519Point} from './src/ed25519Point'
export { default as CP256Point} from './src/cP256Point'
export { default as Hash } from './src/hash';
export { default as ElGamal } from './src/elGamal';
export { default as ElGamalKey } from './src/elGamalKey';
export { default as C25519Key } from './src/c25519Key';
export { default as ed25519Key } from './src/ed25519Key';
export { default as CP256Key } from './src/cP256Key';
export { default as C25519Cipher } from './src/c25519Cipher';
export { default as AesSherableKey } from './src/aesSherableKey';
export { default as EcKeyFormat } from './src/EcKeyFormat';
export { default as CompositeKey } from './src/compositeKey';
export { default as CompositeCipher } from './src/compositeCipher';

import * as BnInput from './src/bnInput';
export { BnInput };

import Hash from './src/hash.js';
import ElGamal from './src/elGamal.js';
import ElGamalKey, { params } from './src/elGamalKey.js';
import SecretShare from './src/secretShare.js';

export default class Index {
    static getKeys(bits = 256, confidence = 32) {
        return ElGamalKey.generate(bits, confidence).map(k => k.toString());
    }

    static shareKey(key, number, threshold) {
        return ElGamalKey.from(key).share(number, threshold).map(k => k.toString());
    }

    static shareText(secret, number, threshold) {
        return SecretShare.shareText(secret, number, threshold, params.p);
    }

    static shareBuffer(secret, number, threshold) {
        return SecretShare.shareBuffer(secret, number, threshold, params.p);
    }

    static encrypt(text, pub) {
        return ElGamal.encrypt(text, ElGamalKey.from(pub));
    }

    static encryptBuffer(text, pub) {
        return ElGamal.encryptBuffer(text, ElGamalKey.from(pub));
    }

    static decrypt(cypher, prv) {
        return ElGamal.decrypt(cypher, ElGamalKey.from(prv));
    }

    static decryptBuffer(cypher, prv) {
        return ElGamal.decryptBuffer(cypher, ElGamalKey.from(prv));
    }

    static decryptPartial(cypher, prv) {
        return ElGamal.decryptPartial(cypher, ElGamalKey.from(prv));
    }

    static decryptPartialBuffer(cypher, prv) {
        return ElGamal.decryptPartialBuffer(cypher, ElGamalKey.from(prv));
    }

    static combineKeys(partials) {
        const keys = partials.map(k => ElGamalKey.from(k));
        return ElGamalKey.combine(keys).toString();
    }

    static combineDeciphers(partials, pub) {
        return ElGamal.combinePartial(partials, ElGamalKey.from(pub));
    }

    static combinePartialBuffer(partials, pub) {
        return ElGamal.combinePartialBuffer(partials, ElGamalKey.from(pub));
    }

    static combineTexts(partials) {
        return SecretShare.interpolateText(partials, params.p);
    }

    static interpolateBuffer(partials) {
        return SecretShare.interpolateBuffer(partials, params.p);
    }

    static hashSha(data) {
        return Hash.sha(data);
    }
}
